import 'package:fis_measure/interfaces/process/calculators/values.dart';
import 'package:fis_measure/process/calcuators/formulas/index.dart' as formulas;
import 'package:fis_measure/process/calcuators/unit_formulas/base/utils.dart';
import 'package:vid/us/vid_us_unit.dart';

import '../base/cardiac.dart';

class CardiacFormulasMouse extends CardiacFormulasBase {
  @override
  UnitFloatValue edvTech(UnitFloatValue lvidd) {
    var lviddV = lvidd.convert(VidUsUnit.mm);
    var value = formulas.CardiacFormulas.edvTeichholz(lviddV);
    return UnitFloatValue(value, VidUsUnit.ml);
  }

  @override
  UnitFloatValue esvTech(UnitFloatValue lvids) {
    var lvidsV = lvids.convert(VidUsUnit.mm);
    var value = formulas.CardiacFormulas.esvTeichholz(lvidsV);
    return UnitFloatValue(value, VidUsUnit.ml);
  }

  @override
  UnitFloatValue ef(UnitFloatValue edv, UnitFloatValue esv) {
    var value = formulas.CardiacFormulas.ef(
      edv.convert(VidUsUnit.mm),
      esv.convert(VidUsUnit.mm),
    );
    return UnitFloatValue(value, VidUsUnit.percent);
  }

  @override
  UnitFloatValue sv(UnitFloatValue edv, UnitFloatValue esv) {
    var value = formulas.CardiacFormulas.sv(
      edv.convert(VidUsUnit.ml),
      esv.convert(VidUsUnit.ml),
    );
    return UnitFloatValue(value, VidUsUnit.ml);
  }

  @override
  UnitFloatValue ci(UnitFloatValue sv, {required int hr, required double bsa}) {
    var value = formulas.CardiacFormulas.ci(
      sv.convert(VidUsUnit.ml),
      hr: hr,
      bsa: bsa,
    );
    return UnitFloatValue(value, VidUsUnit.mlmincm2);
  }

  @override
  UnitFloatValue co(UnitFloatValue sv, {required int hr}) {
    var value = formulas.CardiacFormulas.co(
      sv.convert(VidUsUnit.cm3),
      hr: hr,
    );
    return UnitFloatValue(value, VidUsUnit.mlmin);
  }
}
