import 'package:fis_common/logger/logger.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/process/primitives/trace.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_curvature_measure.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_curvature_trace_measure.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:vid/us/vid_us_unit.dart';
import '../calculator.dart';

/// 曲线弯曲度计算
class URMCurvatureTraceMeasureCal extends Calculator<Trace, double> {
  URMCurvatureTraceMeasureCal(
    Trace ref,
  ) : super(ref);
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;

      // 添加入参
      List<UrmPoint> srcDPoints = [];
      for (var point in ref.feature!.innerPoints) {
        srcDPoints.add(urmApplication.localToView(point));
      }

      URMMeasureProcessResult? result =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMCurvature,
        rOIType: URMROIType.URMTrace,
        srcDPoints: srcDPoints,
      );
      if (result != null) {
        print(
            "URM Measure curvature: ${result.resultData} nums: ${result.resultDPoints?.length}");
        final feature = ref.feature!;
        if (feature is! TraceCurvatureFeature) return;
        feature.autoLinePoints =
            urmApplication.urmPointsToDPoints(result.resultDPoints);

        ItemMeta? meta = feature.refItem.meta;
        var revert = meta.name == URMCurvatureMeasure.revertRatioName;
        if (revert) {
          for (var output in ref.meta.outputs) {
            if (output.name == MeasureTerms.URMCurvature) {
              output.unit = VidUsUnit.None;
              feature.updateFloatValue(
                  output, 1 / result.resultData, output.unit);
            }
          }
        } else {
          for (var output in ref.meta.outputs) {
            if (output.name == MeasureTerms.URMCurvature) {
              output.unit = VidUsUnit.None;
              feature.updateFloatValue(output, result.resultData, output.unit);
            }
          }
        }
        // for (var output in ref.meta.outputs) {
        //   if (output.name == MeasureTerms.URMCurvature) {
        //     output.unit = VidUsUnit.None;
        //     feature.updateFloatValue(output, result.resultData, output.unit);
        //   }
        // }
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }
}
