import 'dart:ui';

import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';

import 'package:fis_measure/process/primitives/urm_measure/urm_rect_measure.dart';
// import 'package:fis_measure/process/primitives/urm_straightline.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';

import '../calculator.dart';

class URMRectMeasureCal extends Calculator<URMRectMeasure, double> {
  URMRectMeasureCal(
    URMRectMeasure ref,
  ) : super(ref);
  URMSimpleMeasureParams? param;
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    if (ref.feature == null) return;
    try {
      Size urmResultSize = const Size(0, 0);
      if (ref.application is! URMApplication) {
        return;
      }
      final URMApplication urmApplication = ref.application as URMApplication;
      urmResultSize = Size(urmApplication.resultWidth.toDouble(),
          urmApplication.resultHeight.toDouble());
      final p1 = ref.feature!.startPoint;
      final p2 = ref.feature!.endPoint;

      String description = "URM\n Calculating...";
      updateStringValue(description);
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
  }

  /// 处理获取到的URM参数
  void handleURMSimpleMeasureParams() {
    if (ref.feature == null) return;
    try {
      if (ref.application is! URMApplication) {
        return;
      }
      final URMApplication urmApplication = ref.application as URMApplication;
      final p1 = ref.feature!.startPoint;
      final p2 = ref.feature!.endPoint;

      param = URMSimpleMeasureParams(
        urmMeasureType: URMMeasureType.URMCurvature,
        rOIType: URMROIType.URMRect,
        srcDPoints: [
          urmApplication.localToView(p1),
          urmApplication.localToView(p2)
        ],
      );
    } catch (e) {
      logger.e('URM handleURMSimpleMeasureParams error: $e');
    }
  }
}
