import 'dart:convert';
import 'dart:typed_data';

import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_rect_measure.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_rect_measure.dart';

// import 'package:fis_measure/process/primitives/urm_straightline.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'dart:ui' as ui;
import 'package:vid/us/vid_us_unit.dart';

class URMRectPerfusionCal extends URMRectMeasureCal {
  URMRectPerfusionCal(URMRectMeasure ref) : super(ref);
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    super.handleURMSimpleMeasureParams();
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;

      URMMeasureProcessResult? outResult =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMPerfusion,
        rOIType: URMROIType.URMRect,
        srcDPoints: param!.srcDPoints,
      );

      if (outResult != null) {
        final feature = ref.feature!;

        for (var output in ref.meta.outputs) {
          if (output.name == MeasureTerms.URMPerfusionIndex) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, outResult.resultData, output.unit);

            if (feature is! RectPerfusionImageFeature) return;
            if (outResult.perfusionImgBase64 != null) {
              feature.perfusionImg =
                  await loadImageFromBase64(outResult.perfusionImgBase64!);
              feature.leftTopPoint = urmApplication
                  .viewToLocal(outResult.perfusionScaleDRect!.topLeft!);
              feature.rightBottomPoint = urmApplication
                  .viewToLocal(outResult.perfusionScaleDRect!.bottomRight!);
              urmApplication.onUpdatePerfusion!(ref.feature!.id);
            }
          }
        }
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }

  Future<ui.Image> loadImageFromBase64(String base64String) async {
    Uint8List bytes = base64Decode(base64String);
    ui.Codec codec = await ui.instantiateImageCodec(bytes);
    ui.FrameInfo frameInfo = await codec.getNextFrame();
    return frameInfo.image;
  }
}
