import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_shell_measure.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_shell_measure.dart';

// import 'package:fis_measure/process/primitives/urm_straightline.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';

class URMShellDensityMeasureCal extends URMShellMeasureCal {
  URMShellDensityMeasureCal(URMShellMeasure ref) : super(ref);
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    super.handleURMSimpleMeasureParams();
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;
      double rimWidth = (ref.meta.rimWidth ?? 0) / 10;
      URMMeasureProcessResult? outresult =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMDen,
        rOIType: URMROIType.URMShell,
        srcDPoints: param!.srcDPoints,
        shellWidth: rimWidth,
      );
      if (outresult == null) {
        return;
      }
      final feature = ref.feature!;
      if (ref.meta.outputs.isNotEmpty) {
        for (var output in ref.meta.outputs) {
          if (output.name == MeasureTerms.URMDenROI) {
            output.unit = VidUsUnit.percent;
            feature.updateFloatValue(
                output, outresult.resultData * 100, output.unit);
          }
        }
        if (feature is! ShellImageFeature) return;
        if (outresult.outerExterPoints != null) {
          List<DPoint> outerExterPoints = [];
          outresult.outerExterPoints?.forEach((element) {
            outerExterPoints.add(urmApplication.viewToLocal(element));
          });
          feature.outerExterPoints = outerExterPoints;
        }
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }
}
