import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/types.dart';
import 'package:fis_measure/interfaces/process/workspace/point_info.dart';
import 'package:fis_measure/process/calcuators/depth2baseline.dart';
import 'package:fis_measure/process/items/top_item.dart';
import 'package:fis_measure/process/items/top_item_feature.dart';
import 'package:fis_measure/process/primitives/location.dart';
import 'package:fis_measure/process/primitives/ray.dart';

class DepthToBaseLine extends TopMeasureItem<DepthToBaseLineFeature> {
  late final Ray baseline;
  late final Location depth;

  DepthToBaseLine(ItemMeta meta) : super(meta) {
    final metaBaseLine = meta.getChildByType(MeasureTypes.Ray)!;
    final metaDepth = meta.getChildByType(MeasureTypes.Distance)!;
    baseline = Ray.createRay(metaBaseLine, this);
    depth = Location.createTissueDepth(metaDepth, this);
    childItems.add(baseline);
    childItems.add(depth);
  }

  @override
  bool get finishAfterUnactive => true;

  @override
  DepthToBaseLineFeature buildFeature() => DepthToBaseLineFeature(this);

  @override
  bool onExecuteMouse(PointInfo args) {
    if (feature == null) {
      feature = buildFeature();
      listenChildrenUpdate();
    }
    if (args.pointType == PointInfoType.mouseDown) {
      if (childrenAllDone) {
        workingChild.clear();
      }
    }
    feature?.hostVisualArea = args.hostVisualArea;
    if (!childrenAllDone) {
      final result = workingChild.execute(args);
      if (result) {
        doCalculate();
      }
      return result;
    }
    return true;
  }

  static DepthToBaseLine createDepthToBaseLine(ItemMeta meta,
      [IMeasureItem? parent]) {
    if (meta.measureType != MeasureTypes.DepthToBaseLine) {
      throw ArgumentError();
    }
    var location = DepthToBaseLine(meta);
    location.calculator = DistanceToRayCal(location);

    return location;
  }
}

class DepthToBaseLineFeature extends TopMeasureItemFeature {
  DepthToBaseLineFeature(
    DepthToBaseLine refItem,
  ) : super(refItem);
}
