import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/process/primitives/multi_method/dop_trace_disp/cardiac_cycle.dart';
import 'package:get/get.dart';

class TraceListData {
  static VetAutoTraceImageResult? _vetAutoTraceImageResult;

  static VetAutoTraceImageResult? get vetAutoTraceImageResult =>
      _vetAutoTraceImageResult;

  static String currentLocationName = "";

  static final List<DPoint> _maxPonints = [];
  static final List<CardiacCycle> _cardiacCycleList = [];

  List<DPoint> get maxPonints => _maxPonints;

  static List<CardiacCycle> get cardiacCycleList => _cardiacCycleList;

  /// 点集
  static List<DPoint> belowMaxPonints = [];
  static List<DPoint> aboveMaxPonints = [];

  /// 心动周期
  static List<CardiacCycle> belowCardiacCycleList = [];
  static List<CardiacCycle> aboveCardiacCycleList = [];

  /// 最佳心动周期
  static List<BestCardiacCycle> belowOptimumCardiacCycleList = [];
  static List<BestCardiacCycle> aboveOptimumCardiacCycleList = [];

  static TraceData? loadPonintsData(String locationName) {
    List<DPoint> points = [];
    List<PWTracePointDTO> contourList = [];
    List<CardiacCycle> cardiacCycleList = [];
    List<BestCardiacCycle> optimumCardiacCycleList = [];
    Map<String, PWTraceDTO>? pWTraceInfos =
        vetAutoTraceImageResult?.pWTraceInfos;
    pWTraceInfos?.forEach((key, value) {
      if (key == locationName) {
        contourList = value.contour ?? [];
        cardiacCycleList = getCardiacCycles(
          value.cycleInfos,
          value.contour,
        );
        optimumCardiacCycleList = getOptimumCardiacCycles(
          value.contour,
          value.bestCycles,
        );
      }
    });

    if (contourList.isEmpty) {
      return null;
    }

    for (int i = 0; i < contourList.length; i++) {
      points.add(
        convert(
          DPoint(
            contourList[i].x.toDouble(),
            contourList[i].y.toDouble(),
          ),
        ),
      );
    }
    return TraceData(
      maxPonints: points,
      cardiacCycle: cardiacCycleList,
      bestCardiacCycle: optimumCardiacCycleList,
    );
  }

  /// 返回的数据很怪
  static List<BestCardiacCycle> getOptimumCardiacCycles(
    List<PWTracePointDTO>? contour,
    PWTraceBestCycleDTO? bestCycles,
  ) {
    List<BestCardiacCycle> cycleLists = [];
    if (bestCycles != null && contour != null) {
      PWTraceBestCycleIndexDTO? oneBestCycle = bestCycles.oneBestCycle;
      PWTraceBestCycleIndexDTO? twoBestCycle = bestCycles.twoBestCycle;
      PWTraceBestCycleIndexDTO? threeBestCycle = bestCycles.threeBestCycle;
      PWTraceBestCycleIndexDTO? fourBestCycle = bestCycles.fourBestCycle;
      PWTraceBestCycleIndexDTO? fiveBestCycle = bestCycles.fiveBestCycle;
      List<PWTraceBestCycleIndexDTO?> bestCycle = [
        oneBestCycle,
        twoBestCycle,
        threeBestCycle,
        fourBestCycle,
        fiveBestCycle
      ];
      for (int i = 0; i < bestCycle.length; i++) {
        if (bestCycle[i] != null) {
          cycleLists.add(BestCardiacCycle(
            bestDiastoleEnd: convert(
              DPoint(
                contour[bestCycle[i]?.bestEnd ?? 0].x.toDouble(),
                contour[bestCycle[i]?.bestEnd ?? 0].y.toDouble(),
              ),
            ),
            bestSystoleStart: convert(
              DPoint(
                contour[bestCycle[i]?.bestStart ?? 0].x.toDouble(),
                contour[bestCycle[i]?.bestStart ?? 0].y.toDouble(),
              ),
            ),
          ));
        }
      }
    }

    return cycleLists;
  }

  static List<CardiacCycle> getCardiacCycles(
      List<PWTraceCycleDTO>? pwTraceCycleLists,
      List<PWTracePointDTO>? contour) {
    List<CardiacCycle> cycleLists = [];
    if (pwTraceCycleLists != null) {
      for (int i = 0; i < pwTraceCycleLists.length; i++) {
        int cycleEnd = pwTraceCycleLists[i].cycleEnd;
        int cycleStart = pwTraceCycleLists[i].cycleStart;
        int epeak = pwTraceCycleLists[i].epeak;
        cycleLists.add(CardiacCycle(
          minimumAbsoluteVelocity: DPoint(0, 0),
          diastoleEnd: convert(
            DPoint(
              contour![cycleEnd].x.toDouble(),
              contour[cycleEnd].y.toDouble(),
            ),
          ),
          index: 0,
          peakSystolic: convert(
            DPoint(
              contour[epeak].x.toDouble(),
              contour[epeak].y.toDouble(),
            ),
          ),
          ePeak: convert(
            DPoint(
              contour[epeak].x.toDouble(),
              contour[epeak].y.toDouble(),
            ),
          ),
          systoleStart: convert(
            DPoint(
              contour[cycleStart].x.toDouble(),
              contour[cycleStart].y.toDouble(),
            ),
          ),
        ));
      }
    }

    return cycleLists;
  }

  /// 转化成逻辑点坐标
  static DPoint convert(
    DPoint point,
  ) {
    /// 像素坐标
    final x = point.x;
    final y = point.y;
    final widthScale = Get.find<IApplication>().displayScaleRatio;
    final displaySize = Get.find<IApplication>().displaySize;

    return DPoint(
      (x * widthScale) / displaySize.width,
      (y * widthScale) / displaySize.height,
    );
  }

  /// 设置自动/半自动数据
  static void setVetAutoTraceImageResult(VetAutoTraceImageResult? result) {
    _vetAutoTraceImageResult = result;

    belowMaxPonints = loadPonintsData("below")?.maxPonints ?? [];
    aboveMaxPonints = loadPonintsData("above")?.maxPonints ?? [];

    belowCardiacCycleList = loadPonintsData("below")?.cardiacCycle ?? [];
    aboveCardiacCycleList = loadPonintsData("above")?.cardiacCycle ?? [];

    belowOptimumCardiacCycleList =
        loadPonintsData("below")?.bestCardiacCycle ?? [];
    aboveOptimumCardiacCycleList =
        loadPonintsData("above")?.bestCardiacCycle ?? [];
  }
}

class TraceData {
  List<DPoint> maxPonints;
  List<CardiacCycle> cardiacCycle;
  List<BestCardiacCycle> bestCardiacCycle;

  TraceData({
    required this.maxPonints,
    required this.cardiacCycle,
    required this.bestCardiacCycle,
  });
}
