import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_ab_ratio.dart';
import 'package:fis_measure/process/items/top_item_feature.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_ellipse_measure.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_rect_measure.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_trace_measure.dart';
import 'package:fis_measure/process/items/item.dart';
import 'package:fis_measure/process/items/item_feature.dart';
import 'package:fis_measure/process/items/top_item.dart';

class URMAbRatioFeature extends TopMeasureItemFeature {
  URMAbRatioFeature(
    ITopMeasureItem refItem,
  ) : super(refItem);
}

abstract class URMAbRatioAbstract<T extends MeasureItemFeature>
    extends TopMeasureItem<T> {
  URMAbRatioAbstract(ItemMeta meta) : super(meta);

  MeasureItem get child1;
  MeasureItem get child2;

  @override
  bool get finishAfterUnactive => true;
}

class TwoURMRectDen extends URMAbRatioAbstract<URMAbRatioFeature> {
  static const String _area1Key = "Roi1";
  static const String _area2Key = "Roi2";

  late final URMRectMeasure a1;
  late final URMRectMeasure a2;

  TwoURMRectDen(ItemMeta meta) : super(meta) {
    final metaA1 = meta.getChildByName(_area1Key)!;
    final metaA2 = meta.getChildByName(_area2Key)!;
    a1 = URMRectMeasure.createURMRectDensity(metaA1, this);
    a2 = URMRectMeasure.createURMRectDensity(metaA2, this);
    childItems.add(a1);
    childItems.add(a2);
  }

  @override
  URMRectMeasure get child1 => a1;

  @override
  URMRectMeasure get child2 => a2;

  @override
  URMAbRatioFeature buildFeature() => URMAbRatioFeature(this);

  @override
  void onCancelingOnce() {}

  static TwoURMRectDen createTwoURMRectDen(ItemMeta meta,
      [IMeasureItem? parent]) {
    var urmTwoURMRectDen = TwoURMRectDen(meta);
    urmTwoURMRectDen.calculator = URMAbRatioCal(urmTwoURMRectDen);

    return urmTwoURMRectDen;
  }
}

class TwoURMTraceDen extends URMAbRatioAbstract<URMAbRatioFeature> {
  static const String _area1Key = "Roi1";
  static const String _area2Key = "Roi2";

  late final URMTraceMeasure a1;
  late final URMTraceMeasure a2;

  TwoURMTraceDen(ItemMeta meta) : super(meta) {
    final metaA1 = meta.getChildByName(_area1Key)!;
    final metaA2 = meta.getChildByName(_area2Key)!;
    a1 = URMTraceMeasure.createURMTraceDensity(metaA1, this);
    a2 = URMTraceMeasure.createURMTraceDensity(metaA2, this);
    childItems.add(a1);
    childItems.add(a2);
  }

  @override
  URMTraceMeasure get child1 => a1;

  @override
  URMTraceMeasure get child2 => a2;

  @override
  URMAbRatioFeature buildFeature() => URMAbRatioFeature(this);

  @override
  void onCancelingOnce() {}

  static TwoURMTraceDen createTwoURMTraceDen(ItemMeta meta,
      [IMeasureItem? parent]) {
    var urmTwoURMRectDen = TwoURMTraceDen(meta);
    urmTwoURMRectDen.calculator = URMAbRatioCal(urmTwoURMRectDen);

    return urmTwoURMRectDen;
  }
}

class TwoURMEllipseDen extends URMAbRatioAbstract<URMAbRatioFeature> {
  static const String _area1Key = "Roi1";
  static const String _area2Key = "Roi2";

  late final URMEllipseMeasure a1;
  late final URMEllipseMeasure a2;

  TwoURMEllipseDen(ItemMeta meta) : super(meta) {
    final metaA1 = meta.getChildByName(_area1Key)!;
    final metaA2 = meta.getChildByName(_area2Key)!;
    a1 = URMEllipseMeasure.createURMEllipseDensity(metaA1, this);
    a2 = URMEllipseMeasure.createURMEllipseDensity(metaA2, this);
    childItems.add(a1);
    childItems.add(a2);
  }

  @override
  URMEllipseMeasure get child1 => a1;

  @override
  URMEllipseMeasure get child2 => a2;

  @override
  URMAbRatioFeature buildFeature() => URMAbRatioFeature(this);

  @override
  void onCancelingOnce() {}

  static TwoURMEllipseDen createTwoURMEllipseDen(ItemMeta meta,
      [IMeasureItem? parent]) {
    var urmTwoURMRectDen = TwoURMEllipseDen(meta);
    urmTwoURMRectDen.calculator = URMAbRatioCal(urmTwoURMRectDen);

    return urmTwoURMRectDen;
  }
}

class TwoURMRectFractal extends URMAbRatioAbstract<URMAbRatioFeature> {
  static const String _area1Key = "Roi1";
  static const String _area2Key = "Roi2";

  late final URMRectMeasure a1;
  late final URMRectMeasure a2;

  TwoURMRectFractal(ItemMeta meta) : super(meta) {
    final metaA1 = meta.getChildByName(_area1Key)!;
    final metaA2 = meta.getChildByName(_area2Key)!;
    a1 = URMRectMeasure.createURMRectFractalDim(metaA1, this);
    a2 = URMRectMeasure.createURMRectFractalDim(metaA2, this);
    childItems.add(a1);
    childItems.add(a2);
  }

  @override
  URMRectMeasure get child1 => a1;

  @override
  URMRectMeasure get child2 => a2;

  @override
  URMAbRatioFeature buildFeature() => URMAbRatioFeature(this);

  @override
  void onCancelingOnce() {}

  static TwoURMRectFractal createTwoURMRectFractal(ItemMeta meta,
      [IMeasureItem? parent]) {
    var urmTwoURMRectDen = TwoURMRectFractal(meta);
    urmTwoURMRectDen.calculator = URMAbRatioCal(urmTwoURMRectDen);

    return urmTwoURMRectDen;
  }
}

class TwoURMTraceFractal extends URMAbRatioAbstract<URMAbRatioFeature> {
  static const String _area1Key = "Roi1";
  static const String _area2Key = "Roi2";

  late final URMTraceMeasure a1;
  late final URMTraceMeasure a2;

  TwoURMTraceFractal(ItemMeta meta) : super(meta) {
    final metaA1 = meta.getChildByName(_area1Key)!;
    final metaA2 = meta.getChildByName(_area2Key)!;
    a1 = URMTraceMeasure.createURMTraceFractalDim(metaA1, this);
    a2 = URMTraceMeasure.createURMTraceFractalDim(metaA2, this);
    childItems.add(a1);
    childItems.add(a2);
  }

  @override
  URMTraceMeasure get child1 => a1;

  @override
  URMTraceMeasure get child2 => a2;

  @override
  URMAbRatioFeature buildFeature() => URMAbRatioFeature(this);

  @override
  void onCancelingOnce() {}

  static TwoURMTraceFractal createTwoURMTraceFractal(ItemMeta meta,
      [IMeasureItem? parent]) {
    var urmTwoURMRectDen = TwoURMTraceFractal(meta);
    urmTwoURMRectDen.calculator = URMAbRatioCal(urmTwoURMRectDen);

    return urmTwoURMRectDen;
  }
}

class TwoURMEllipseFractal extends URMAbRatioAbstract<URMAbRatioFeature> {
  static const String _area1Key = "Roi1";
  static const String _area2Key = "Roi2";

  late final URMEllipseMeasure a1;
  late final URMEllipseMeasure a2;

  TwoURMEllipseFractal(ItemMeta meta) : super(meta) {
    final metaA1 = meta.getChildByName(_area1Key)!;
    final metaA2 = meta.getChildByName(_area2Key)!;
    a1 = URMEllipseMeasure.createURMEllipseFractalDim(metaA1, this);
    a2 = URMEllipseMeasure.createURMEllipseFractalDim(metaA2, this);
    childItems.add(a1);
    childItems.add(a2);
  }

  @override
  URMEllipseMeasure get child1 => a1;

  @override
  URMEllipseMeasure get child2 => a2;

  @override
  URMAbRatioFeature buildFeature() => URMAbRatioFeature(this);

  @override
  void onCancelingOnce() {}

  static TwoURMEllipseFractal createTwoURMEllipseFractal(ItemMeta meta,
      [IMeasureItem? parent]) {
    var urmTwoURMRectDen = TwoURMEllipseFractal(meta);
    urmTwoURMRectDen.calculator = URMAbRatioCal(urmTwoURMRectDen);

    return urmTwoURMRectDen;
  }
}
