import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/enums/items.dart';
import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/workspace/point_info.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_curve_curvature_line.dart';
import 'package:fis_measure/process/primitives/area_abstract.dart';
import 'package:fis_measure/process/primitives/trace.dart';
import 'package:fis_measure/utils/canvas.dart';
import 'package:fis_measure/utils/prompt_box.dart';
import 'package:flutter/material.dart';

class URMCurveCurvatureLineMeasure extends Trace {
  URMCurveCurvatureLineMeasure(ItemMeta meta, IMeasureItem? parent)
      : super(meta, parent);

  static URMCurveCurvatureLineMeasure createURMCurveCurvatureLineMeasure(
      ItemMeta meta,
      [IMeasureItem? parent]) {
    URMCurveCurvatureLineMeasure trace =
        URMCurveCurvatureLineMeasure(meta, parent);
    trace.calculator = URMCurveCurvatureLineCal(trace);
    trace.isClosed = false;
    return trace;
  }

  @override
  bool onExecuteMouse(PointInfo args) {
    isAutoSnap = false;
    if (waitingResult) return false;
    if (state == ItemStates.finished) {
      if (args.pointType == PointInfoType.mouseDown) {
        state = ItemStates.waiting;
      }
    }

    if (state == ItemStates.waiting) {
      if (args.pointType == PointInfoType.mouseDown) {
        handleMouseDownWhileWaiting(args);
      }
    } else if (state == ItemStates.running) {
      if (args.pointType == PointInfoType.mouseUp) return false;

      feature?.adopt(args);
      if (args.pointType == PointInfoType.mouseDown) {
        handleFinish();
      } else {
        checkAutoSnap(args);
      }
    }
    return true;
  }

  @override
  void handleMouseDownWhileWaiting(PointInfo args) {
    feature = TraceCurvatureLineFeature(this);
    if (args.hostVisualArea != null) {
      feature!.hostVisualArea = args.hostVisualArea;
    }
    final point = args.toAreaLogicPoint();
    feature!.adopt(point);
    state = ItemStates.running;
  }

  bool waitingResult = false;

  void handleFinish() async {
    feature!.isActive = false;
    waitingResult = true;
    await doCalculateAsync();
    doFeatureFinish();
    doFeatureUpdate(); // 若不执行，子测量将无法自动切换
    PromptBox.dismiss();
    waitingResult = false;
  }
}

class TraceCurvatureLineFeature extends TraceFeature {
  TraceCurvatureLineFeature(AreaItemAbstract refItem) : super(refItem);

  List<DPoint> autoLinePoints = [];

  @override
  void paint(Canvas canvas, Size size) {
    super.paint(canvas, size);

    /// 绘制 AutoLine 点集连线
    /// 使用 convert2ViewPoint 转为像素坐标
    final pixelPoints = autoLinePoints.map((e) {
      return convert2ViewPoint(size, e).toOffset();
    }).toList();
    final greenLinePan = Paint()
      ..color = Colors.green
      ..isAntiAlias = true
      ..strokeWidth = 2
      ..style = PaintingStyle.stroke;
    canvas.drawPointsLine(pixelPoints, greenLinePan);
  }
}
