// ignore_for_file: must_be_immutable

import 'dart:convert';

import 'package:collection/collection.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/define.dart';
import 'package:fis_measure/view/ai_result_modifier/controller.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:fis_theme/theme.dart';

/// ai特征分析
class AiFeatureAnalysis extends FisView<AiResultModifierController> {
  List<AIDiagnosisDescription>? descriptions;

  final _scrollController = ScrollController();
  AiFeatureAnalysis(this.descriptions, {super.key});
  @override
  FWidget build(BuildContext context) {
    return FContainer(
      child: FColumn(
        children: [
          FRow(
            mainAxisSize: MainAxisSize.max,
            children: [
              FExpanded(
                child: FContainer(
                  decoration: const BoxDecoration(
                    borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(4),
                      topRight: Radius.circular(4),
                    ),
                    color: Color.fromRGBO(54, 169, 206, 1),
                  ),
                  padding: const EdgeInsets.symmetric(
                    vertical: 4,
                    horizontal: 8,
                  ),
                  child: FText(
                    i18nBook.measure.featureAnalysis.t,
                    style: const TextStyle(
                      color: Colors.white,
                    ),
                  ),
                ),
              )
            ],
          ),
          FExpanded(
            child: _aiFeatureAnalysisFScrollbar(),
          ),
        ],
      ),
    );
  }

  FWidget _aiFeatureAnalysisFScrollbar() {
    return FScrollbar(
      isAlwaysShown: true,
      controller: _scrollController,
      child: FGridView.count(
        shrinkWrap: true,
        crossAxisCount: 2,
        childAspectRatio: 2,
        controller: _scrollController,
        children: descriptions!
            .where((e) => e.type != DiagnosisDescriptionEnum.LesionSize)
            .mapIndexed(
          (index, e) {
            return FContainer(
              decoration: BoxDecoration(
                border: Border.all(
                  color: const Color.fromRGBO(54, 169, 206, 1),
                  width: 1,
                ),
              ),
              padding: const EdgeInsets.all(4),
              child: FColumn(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  FContainer(
                    margin: const EdgeInsets.only(
                      top: 5,
                      left: 5,
                    ),
                    child: FTooltip(
                      message: _buildDescriptionType(e.type,
                          isDiagnosisDescriptionTitle: false),
                      child: FText(
                        _buildDescriptionType(e.type,
                            isDiagnosisDescriptionTitle: false),
                        style: const TextStyle(
                          color: Color.fromRGBO(54, 169, 206, 1),
                        ),
                        overflow: TextOverflow.ellipsis,
                        maxLines: 1,
                      ),
                      textLines: const [],
                    ),
                  ),
                  FExpanded(
                    child: FCenter(
                      child: FContainer(
                        margin: const EdgeInsets.symmetric(horizontal: 5),
                        child: _aiResultValue(index, e),
                      ),
                    ),
                  ),
                ],
              ),
            );
          },
        ).toList(),
      ),
    );
  }

  FWidget _aiResultValue(
    int aiResultindex,
    AIDiagnosisDescription aiDiagnosisDescription,
  ) {
    List<FSelectOptionModel<String>> source =
        _initEnumFieldList(aiResultindex, aiDiagnosisDescription.type);
    return FSelect<FSelectOptionModel<String>, String>(
      height: 38,
      source: source,
      hintText: i18nBook.common.Select.t,
      fontFamily: FTheme.ins.localeSetting.fontFamily,
      optionValueExtractor: (FSelectOptionModel<String> data) => data.value,
      optionLabelExtractor: (data) => data.title,
      value: _selectValidator(
          jsonEncode(AiFeatureAnalysisResult(
            currentIndex: aiResultindex,
            resultValue: aiDiagnosisDescription.value ?? '',
          ).toJson()),
          source),
      textColor: Colors.white,
      onSelectChanged: (value, index) {
        descriptions?[jsonDecode(value!)?['currentIndex'] ?? 0].value =
            jsonDecode(value)!['resultValue'];
      },
    );
  }

  String _buildDescriptionType(
    DiagnosisDescriptionEnum diagnosisDescriptionType, {
    bool isDiagnosisDescriptionTitle = true,
  }) {
    final Map<DiagnosisDescriptionEnum, String> titleMap = {
      DiagnosisDescriptionEnum.Shape: 'Shape',
      DiagnosisDescriptionEnum.Orientation: 'Orientation',
      DiagnosisDescriptionEnum.EchoPattern: 'EchoPattern',
      DiagnosisDescriptionEnum.LesionBoundary: 'LesionBoundary',
      DiagnosisDescriptionEnum.Margin: 'Margin',
      DiagnosisDescriptionEnum.Calcification: 'Calcification',
      DiagnosisDescriptionEnum.ThyroidEchoPattern: 'ThyroidEchoPattern',
      DiagnosisDescriptionEnum.ThyroidShape: 'ThyroidShape',
      DiagnosisDescriptionEnum.ThyroidMargin: 'ThyroidMargin',
      DiagnosisDescriptionEnum.ThyroidEchogenicFoci: 'ThyroidEchogenicFoci',
      DiagnosisDescriptionEnum.LiverShape: 'LiverShape',
      DiagnosisDescriptionEnum.LiverBoundary: 'LiverBoundary',
      DiagnosisDescriptionEnum.LiverEchoTexture: 'LiverEchoTexture',
      DiagnosisDescriptionEnum.LesionSize: 'LesionSize',
    };
    final Map<DiagnosisDescriptionEnum, String> descMap = {
      DiagnosisDescriptionEnum.Shape: i18nBook.measure.shape.t,
      DiagnosisDescriptionEnum.Orientation: i18nBook.measure.orientation.t,
      DiagnosisDescriptionEnum.EchoPattern: i18nBook.measure.echoPattern.t,
      DiagnosisDescriptionEnum.LesionBoundary:
          i18nBook.measure.lesionBoundary.t,
      DiagnosisDescriptionEnum.Margin: i18nBook.measure.margin.t,
      DiagnosisDescriptionEnum.Calcification: i18nBook.measure.calcification.t,
      DiagnosisDescriptionEnum.ThyroidEchoPattern:
          i18nBook.measure.echoPattern.t,
      DiagnosisDescriptionEnum.ThyroidShape: i18nBook.measure.shape.t,
      DiagnosisDescriptionEnum.ThyroidMargin: i18nBook.measure.margin.t,
      DiagnosisDescriptionEnum.ThyroidEchogenicFoci:
          i18nBook.measure.thyroidEchogenicFoci.t,
      DiagnosisDescriptionEnum.LiverShape: i18nBook.measure.shape.t,
      DiagnosisDescriptionEnum.LiverBoundary: i18nBook.measure.lesionBoundary.t,
      DiagnosisDescriptionEnum.LiverEchoTexture:
          i18nBook.measure.liverEchoTexture.t,
      DiagnosisDescriptionEnum.LesionSize: i18nBook.measure.lesionSize.t,
    };
    const String defaultTitle = 'Shape';
    const String defaultDesc = '';

    if (isDiagnosisDescriptionTitle) {
      return titleMap[diagnosisDescriptionType] ?? defaultTitle;
    } else {
      return descMap[diagnosisDescriptionType] ?? defaultDesc;
    }
  }

  List<FSelectOptionModel<String>> _initEnumFieldList(
    int aiResultindex,
    DiagnosisDescriptionEnum type,
  ) {
    List<FSelectOptionModel<String>> source = [];
    if (controller.diagnosisEnumItems.isNotEmpty) {
      List<EnumFieldDTO> enumFieldList =
          controller.diagnosisEnumItems.firstWhereOrNull((element) {
                return element.code == _buildDescriptionType(type);
              })?.children ??
              [];
      for (int i = 0; i < enumFieldList.length; i++) {
        source.add(
          FSelectOptionModel<String>(
            title: controller
                .getValuesFromAiLanguage(enumFieldList[i].value ?? ''),
            value: jsonEncode(AiFeatureAnalysisResult(
              currentIndex: aiResultindex,
              resultValue: enumFieldList[i].value ?? '',
            ).toJson()),
          ),
        );
      }
    }
    return source;
  }

  /// 内容校验
  String? _selectValidator(
      String value, List<FSelectOptionModel<String>> source) {
    // 判断 value 是否存在与 source 中
    if (source.every((element) => element.value != value)) {
      return null;
    } else {
      return value;
    }
  }
}

class AiFeatureAnalysisResult {
  final String resultValue;

  final int currentIndex;
  AiFeatureAnalysisResult({
    required this.resultValue,
    required this.currentIndex,
  });

  factory AiFeatureAnalysisResult.fromJson(Map<String, dynamic> map) {
    return AiFeatureAnalysisResult(
      resultValue: map['resultValue'],
      currentIndex: map['currentIndex'],
    );
  }

  Map<String, dynamic> toJson() {
    Map<String, dynamic> map = {};
    map['resultValue'] = resultValue;
    map['currentIndex'] = currentIndex;
    return map;
  }
}
