import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/view/player/controller.dart';
import 'package:fis_measure/view/player/events.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

///单独的播放器控制按钮
class SinglePlayButton extends StatefulWidget {
  const SinglePlayButton({Key? key}) : super(key: key);
  @override
  State<StatefulWidget> createState() => SinglePlayButtonState();
}

class SinglePlayButtonState extends State<SinglePlayButton> {
  final playerController = Get.find<IPlayerController>() as VidPlayerController;

  @override
  void initState() {
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      if (mounted) {
        playerController.eventHandler.addListener(onControllerEvent);
      }
    });
    super.initState();
  }

  @override
  void dispose() {
    playerController.eventHandler.removeListener(onControllerEvent);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final playing = playerController.playing;
    const iconSize = 36.0;
    return IconButton(
      splashRadius: 2,
      padding: EdgeInsets.zero,
      onPressed: () {
        if (playing) {
          playerController.locateTo(playerController.currentFrameIndex);
          // playerController.gotoFrame(playerController.currentFrameIndex);
        } else {
          playerController.play();
        }
      },
      icon: playing
          ? const Icon(
              Icons.pause_circle_outline,
              size: iconSize,
              color: Colors.white,
            )
          : const Icon(
              Icons.play_circle_outline,
              size: iconSize,
              color: Colors.white,
            ),
    );
  }

  void onControllerEvent(Object sender, VidPlayerEvent e) {
    if (e is VidPlayerStatusChangeEvent) {
      onPlayStatusChanged(e);
    }
  }

  void onPlayStatusChanged(VidPlayerStatusChangeEvent e) {
    if (mounted) {
      setState(() {});
    }
  }
}
